@@ -8,7 +8,7 @@ import json
 from threading import Thread
 import colorsys
 import os
-from typing import Any, List, Dict, Tuple, Union, Callable
+from typing import Any, List, Dict, Tuple, Union, Callable, Optional
 from sseclient import SSEClient
 import requests
 
@@ -45,6 +45,7 @@ class Nanoleaf():
         :type print_errors: bool
         """
         self.ip = ip
+        self.print_errors = print_errors
         self.url = "http://" + ip + ":16021/api/v1/" + str(auth_token)
         self.check_connection()
         if auth_token is None:
@@ -54,7 +55,6 @@ class Nanoleaf():
         else:
             self.auth_token = auth_token
         self.url = "http://" + ip + ":16021/api/v1/" + str(self.auth_token)
-        self.print_errors = print_errors
         self.already_registered = False
 
 
@@ -99,10 +99,16 @@ class Nanoleaf():
         """
         file_path = os.path.expanduser('~') + os.path.sep + '.nanoleaf_token'
         if os.path.exists(file_path) is False:
-            open(file_path, 'w')
-        token = open(file_path, 'r').read()
-        if token:
-            return token
+            with open(file_path, 'w', encoding='utf-8'):
+                pass
+        with open(file_path, 'r', encoding='utf-8') as token_file:
+            tokens = token_file.readlines()
+        for token in tokens:
+            if token != "":
+                token = token.rstrip()
+                response = requests.get("http://" + self.ip + ":16021/api/v1/" + str(token))
+                if self.__error_check(response.status_code):
+                    return token
 
         response = requests.post('http://' + self.ip + ':16021/api/v1/new')
 
@@ -111,11 +117,13 @@ class Nanoleaf():
             data = json.loads(response.text)
 
             if 'auth_token' in data:
-                open(file_path, 'w').write(data['auth_token'])
+                with open(file_path, 'a', encoding='utf-8') as file:
+                    file.write("\n" + data['auth_token'])
                 return data['auth_token']
         return None
 
-    def delete_auth_token(self, auth_token : str =None) -> bool:
+
+    def delete_auth_token(self, auth_token : str) -> bool:
         """Deletes an authentication token
 
         Deletes an authentication token and the .nanoleaf_token file if it
@@ -123,20 +131,11 @@ class Nanoleaf():
         as part of an API call to control the device. If required, generate
         a new one using create_auth_token().
 
-        :param auth_token: Optional, the authentication token to delete, otherwise
-            delete currently initialised one
+        :param auth_token: The authentication token to delete.
 
         :returns: True if successful, otherwise False
         """
-        file_path = os.path.expanduser('~') + os.path.sep + '.nanoleaf_token'
-        if os.path.exists(file_path):
-            token = open(file_path, 'r').read()
-            if (auth_token is None and self.auth_token == token) or (auth_token == token):
-                os.remove(file_path)
-        if auth_token is None:
-            url = "http://" + self.ip + ":16021/api/v1/" + str(self.auth_token)
-        else:
-            url = "http://" + self.ip + ":16021/api/v1/" + str(auth_token)
+        url = "http://" + self.ip + ":16021/api/v1/" + str(auth_token)
         response = requests.delete(url)
         return self.__error_check(response.status_code)
 
@@ -156,8 +155,8 @@ class Nanoleaf():
         """Returns the name of the current device"""
         return self.get_info()['name']
 
-    def get_auth_token(self) -> str:
-        """Returns the current auth token"""
+    def get_auth_token(self) -> Optional[str]:
+        """Returns the current auth token or None"""
         return self.auth_token
 
     def get_ids(self) -> List[int]:
@@ -496,10 +495,11 @@ class Nanoleaf():
         anim_data = str(len(ids))
         frame_string = ""
         for device_id in ids:
-            frame_string += " {id} 2".format(id=device_id)
+            frame_string += f" {device_id} 2"
             r, g, b = rgb[0], rgb[1], rgb[2]
-            frame_string += " {r} {g} {b} 0 {speed} 0 0 0 0 {speed_2}".format(
-                    r=r, g=g, b=b, speed=int(speed*10), speed_2=int(speed*10))
+            speed = int(speed*10)
+            speed_2 = int(speed*10)
+            frame_string += f" {r} {g} {b} 0 {speed} 0 0 0 0 {speed_2}"
         base_effect['animData'] = anim_data + frame_string
         return self.write_effect(base_effect)
 
@@ -533,10 +533,12 @@ class Nanoleaf():
         anim_data = str(len(ids))
         frame_string = ""
         for device_id in ids:
-            frame_string += " {id} {numFrames}".format(id=device_id, numFrames=len(rgb_list))
+            number_frames = len(rgb_list)
+            frame_string += f" {device_id} {number_frames}"
             for rgb in rgb_list:
                 r, g, b = rgb[0], rgb[1], rgb[2]
-                frame_string += " {r} {g} {b} 0 {speed}".format(r=r, g=g, b=b, speed=int(speed*10))
+                speed = int(speed*10)
+                frame_string += f" {r} {g} {b} 0 {speed}"
         base_effect['animData'] = anim_data + frame_string
         return self.write_effect(base_effect)
 
@@ -558,14 +560,26 @@ class Nanoleaf():
         anim_data = str(len(ids))
         frame_string = ""
         for device_id in ids:
-            frame_string += " {id} {numFrames}".format(id=device_id,
-                numFrames=len(spectrum_palette))
+            number_frames = len(spectrum_palette)
+            frame_string += f" {device_id} {number_frames}"
             for rgb in spectrum_palette:
                 r, g, b = rgb[0], rgb[1], rgb[2]
-                frame_string += " {r} {g} {b} 0 {speed}".format(r=r, g=g, b=b, speed=int(speed*10))
+                speed = int(speed*10)
+                frame_string += f" {r} {g} {b} 0 {speed}"
         base_effect['animData'] = anim_data + frame_string
         return self.write_effect(base_effect)
 
+    def enable_extcontrol(self) -> bool:
+        """Enables the extControl UDP streaming mode
+
+        :returns: True if successful, otherwise False
+        """
+        data = {"write": {"command":  "display",
+                        "animType": "extControl",
+                        "extControlVersion": "v2"}}
+        response = requests.put(self.url + "/effects", data=json.dumps(data))
+        return self.__error_check(response.status_code)
+
     #######################################################
     ####                  LAYOUT                       ####
     #######################################################
